﻿namespace ProjectDumpTruck.Entities;

public class EntityDumpTruck : EntityTruck
{

    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>
    public Color AdditionalColor { get; private set; }

    /// <summary>
    /// Метод передачи дополнительного цвета
    /// </summary>
    /// <param name="additionalColor"></param>
    public void SetAdditionalColor(Color additionalColor)
    {
        AdditionalColor = additionalColor;
    }

    /// <summary>
    /// Признак (опция) наличие кузова
    /// </summary>
    public bool Awning { get; private set; }
    /// <summary>
    /// Признак (опция) наличия тента
    /// </summary>
    public bool Tent { get; private set; }

    /// <summary>
    /// Инициализация полей объекта-класса самосвала
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="awning">Признак наличия обвеса</param>
    /// <param name="tent">Признак наличия антикрыла</param>
    public EntityDumpTruck(int speed, double weight, Color bodyColor, Color
    additionalColor, bool awning, bool tent) : base(speed, weight, bodyColor)
    {
        AdditionalColor = additionalColor;
        Awning = awning;
        Tent = tent;
    }

    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityDumpTruck), Speed.ToString(), Weight.ToString(), BodyColor.Name, AdditionalColor.Name, Awning.ToString() , Tent.ToString() };
    }

    public static EntityDumpTruck? CreateEntityDumpTruck(string[] strs)
    {
        if (strs.Length != 7 || strs[0] != nameof(EntityDumpTruck)) return null;
        
        return new EntityDumpTruck(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]),
            Color.FromName(strs[4]), Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]));
    }
}
